from Grafo import Graph
import BuildGraph
from pprint import pprint

'''
    Funzione che restituisce la matrice delle adiacenze che corrisponde
    ad un grafo g.
'''
def matad(g):
    mat = []
    vertlist = list(g.getVertices())
    vertlist.sort()
    for i in range(len(vertlist)):
        vi = vertlist[i]
        riga = []
        for j in range(len(vertlist)):
            vj = vertlist[j]
            # Se i vertici vi e vj sono collegati la matrice
            # deve riportare alla posizione (i,j) il peso
            # dell'arco corrispondente, altrimento 0
            if vj in g.getVertex(vi).getConnections():
                peso = g.getVertex(vi).getWeight(vj)
                riga.append(peso)
            else:
                riga.append(0) 
        mat.append(riga)
    return mat

'''
    Calcolo del grado uscente del vertice v
    (numero degli archi che partono da v)
'''
def gradouscente(g,v):
    grado = len(g.getVertex(v).getConnections())
    return grado

'''
    Calcolo del grado entrante del vertice v
    (numero degli archi che arrivano a v)
'''
def gradoentrante(g,v):
    grado = 0
    for vert in g.getVertices():
        if v in g.getVertex(vert).getConnections():
            grado += 1
    return grado

'''
    Calcolo del costo minimo degli archi che partono da un vertice v
    basato sulla ricerca in ampiezza.
'''
def trovaMinimo(g,v):
    coda = [v]
    visitati = []
    minimo = None
    while len(coda) > 0:
        corr = coda.pop(0)
        for vid in g.getVertex(corr).getConnections():
            if vid not in visitati and vid not in coda:
                coda.append(vid)
                # Si considera il costo (peso) dell'arco che
                # parte dal vertice corr e arriva al vertice vid.
                # Se il valore minimo non è stato già definito oppure
                # è maggiore del peso corrente, allora viene aggiornato.
                peso = g.getVertex(corr).getWeight(vid)
                if minimo is None:
                    minimo = peso
                elif minimo > peso:
                    minimo = peso
        visitati.append(corr)
    return minimo
    
g = BuildGraph.costruisci_grafo("archi2.txt")
v = 'V5'
print("Matrice delle adiacenze del grafo")
pprint(matad(g))
print("Grado uscente del vertice",v,":",gradouscente(g,v))
print("Grado entrante del vertice",v,":",gradoentrante(g,v))
print("Costo minimo degli archi che partono da",v+':',trovaMinimo(g,'V5'))
